from libc.math cimport sin, cos, sqrt, acos, sinh, cosh

def flatdistsq (double[:] x, double[:] y):
    cdef double res = 0.0
    cdef double s1=x[0]
    cdef double s2=y[0]
    cdef double ra1=x[1]
    cdef double ra2=y[1]
    cdef double dec1=x[2]
    cdef double dec2=y[2]
    cdef double costheta=sin(dec1)*sin(dec2)+cos(dec1)*cos(dec2)*cos(ra1-ra2)
    res= s1**2+s2**2-2.0*s1*s2*costheta
    return res

def opendistsq (double[:] x, double[:] y):
    cdef double res = 0.0
    cdef double K = -1.0
    cdef double s1=sinh(x[0]*0.007)
    cdef double s2=sinh(y[0]*0.007)
    cdef double c1=cosh(x[0]*0.007)
    cdef double c2=cosh(y[0]*0.007)
    cdef double ra1=x[1]
    cdef double ra2=y[1]
    cdef double dec1=x[2]
    cdef double dec2=y[2]
    cdef double costheta=sin(dec1)*sin(dec2)+cos(dec1)*cos(dec2)*cos(ra1-ra2)
    res= s1*s1+s2*s2-2.0*s1*s2*c1*c2*costheta-K*s1*s1*s2*s2*sqrt(1.0+costheta*costheta)
    return res*20408.1633

def closedistsq (double[:] x, double[:] y):
    cdef double res = 0.0
    cdef double K = 1.0
    cdef double s1=sin(x[0]*0.007)
    cdef double s2=sin(y[0]*0.007)
    cdef double c1=cos(x[0]*0.007)
    cdef double c2=cos(y[0]*0.007)
    cdef double ra1=x[1]
    cdef double ra2=y[1]
    cdef double dec1=x[2]
    cdef double dec2=y[2]
    cdef double costheta=sin(dec1)*sin(dec2)+cos(dec1)*cos(dec2)*cos(ra1-ra2)
    res= s1*s1+s2*s2-2.0*s1*s2*c1*c2*costheta-K*s1*s1*s2*s2*sqrt(1.0+costheta*costheta)
    return res*20408.1633

def APzdth(double[:] x, double[:] y):
    return (x[0]+y[0])*0.5*acos(sin(x[2])*sin(y[2])+cos(x[2])*cos(y[2])*cos(x[1]-y[1]))

def APdz(double[:] x, double[:] y):
    return abs(x[0]-y[0])



# from fileios import *
# msg = 'Enter Absolute Path to file: '
# f_name = raw_input(msg).strip()
#
# path = file_data_and_path(f_name)
# if path != None:
#        print 'Path:',path
# from Tkinter import Tk
# from tkFileDialog import askopenfilename
#
# Tk().withdraw() # we don't want a full GUI, so keep the root window from appearing
# filename = askopenfilename() # show an "Open" dialog box and return the path to the selected file
# print(filename)
# def get_filename(file_type):
#     while True:
#         print('enter ' + file_type + ' filename: ')
#         filename = input()
#         print(filename)
#         try:
#             with open(filename, 'r') as f:
#                 my_file = f.read()
#             return my_file
#         except FileNotFoundError:
#             print('No such file.  Check file name and path and try again.')
#
#
# x = get_filename('TEMPLATE')
# print(x)
# -*- coding: utf-8 -*-
"""To add test methods.
"""
# from time import sleep
# from halo import Halo
# from time import time
#
# def rocket_launch():
#     #spinner = Halo({'spinner': 'shark'})
#     spinner = Halo({
#     'spinner': {
#         'interval': 100,
#         'frames': ['-', '\\', '|', '/', '-']
#     }
# })
#     spinner.start()
#     while(1):
#         spinner.text = 'Running... Time Elapsed: {} seconds'.format(time())
#         sleep(10)
#         break
#     spinner.succeed('Rocket launched')
#
# rocket_launch()

from antpcf import *

bins = np.arange(0.01, 0.201, 0.01)
# atpcf('/Users/rohin/Downloads/DR7-Full.ascii', bins, randfile='/Users/rohin/Downloads/random-DR7-Ful.ascii',permetric='apzdth', parmetric='apdz', weights=True)
# tpcf('/Users/rohin/Downloads/DR3-ns.ascii',bins,randfile='/Users/rohin/Downloads/random-DR3-ns.ascii',weights=True)
# def pmethod():
# bins = np.arange(0.002, 0.06, 0.002)
# correl = tpcf('./testw.dat',bins,randfile='./testw.dat',weights=True)
#    return correl
# pool = multiprocessing.Pool(processes=ncount)
# correl = pool.map(pmethod)
# print correl
atpcf('./testw.dat',bins,randfile='./testw.dat',permetric='apzdth',parmetric='apdz',method='ls',weights=True)
# blha=readfitsfile('/Users/rohin/Documents/ipy_notebooks/galsurveystudy/input/galaxy_DR12v5_CMASS_North.fits','data')
# dr12gcmn, weights = datprep('/Users/rohin/Documents/ipy_notebooks/galsurveystudy/input/galaxy_DR12v5_LOWZ_South.fits','data','lcdm')
# dat = readfitsfile('/Users/rohin/Documents/ipy_notebooks/galsurveystudy/input/galaxy_DR12v5_LOWZ_South.fits','data')
# weights = dat['WEIGHT_SYSTOT']
# import pyfits
# dpy = pyfits.open('/Users/rohin/Documents/ipy_notebooks/galsurveystudy/input/galaxy_DR12v5_LOWZ_South.fits')
# dpyd = dpy[1].data
# wts = dpyd['WEIGHT_SYSTOT']
# print(wts)
# print(min(wts))
# print(max(wts))
# print(dr12gcmn)
# print(weights)
# print (min(weights))
# print(max(weights))
# dr12gls=tpcf('/Users/rohin/Documents/ipy_notebooks/galsurveystudy/input/galaxy_DR12v5_LOWZ_South.fits',bins,randfile='/Users/rohin/Downloads/random0_DR12v5_LOWZ_South.fits',weights=True)



import healpix_util as hu
import numpy as np
# import matplotlib.pyplot as plt


def healpixmap(ra, dec, weights):
    """Method to view/visualize angular distribution of galaxies in terms of healpix maps"""
    NSIDE = 512
    hpix = hu.HealPix("ring", NSIDE)
    pix = hpix.eq2pix(ra, dec)
    hpixdata = np.array(np.zeros(hu.nside2npix(NSIDE)))
    for j in range(len(pix)):
        hpixdata[pix[j]] += weights[j]
    hu.mollview(hpixdata, rot=180)
    return hpixdata



# -*- coding: utf-8 -*-
"""Update runtime graphic.
"""
from time import sleep
from halo import Halo
from time import time

#def runupdate(text,fintext):
    #spinner = Halo({'spinner': 'shark'})
spinner = Halo({
    'spinner': {
        'interval': 100,
        'frames': ['-', '\\', '|', '/', '-']
    }    })

#spinner.text = text  #'Running...'# Time Elapsed: {} seconds'.format(time())
#spinner.start()
#spinner.succeed(fintext)





from __future__ import division
# Samsiddhir Haritoshanam!
__author__ = 'Rohin Kumar Y'

# tpcf(dat, datR=None, randcatsize=2, bins,**kwargs)
#
# **kwargs for choosing geometry - metric 'flat' 'open' 'close'
# **kwargs for choosing xi estimator - 'simple' 'ls' '...'
# import fileios
from tqdm import *
from datprep import *
import numpy as np
from metrics import *
from multiprocessing import cpu_count
from multiprocessing import Process
from multiprocessing.queues import Queue
from sklearn.neighbors import BallTree
from scipy.spatial import distance as dist

pcpus=cpu_count()-1


def tpcf(datfile, bins, **kwargs):
    """Main function to calculate 2pCF. Takes multiple arguments such as randfile, maskfile, calculation estimator etc. for different geometry, cosmology models
    Usage of the package is given in jupyter notebook "Using correlcalc example.nb" and in `main.py`

    All the methods in correlcalc can be imported using the following command

    `from correlcalc import *`

    We first need to define bins (in $c/H_0$ units) to calculate 2pCF. For e.g. to calculate correlation between 0-180Mpc in steps of 6Mpc, we say

    `bins=np.arange(0.002,0.06,0.002)`

    To calculate 2pCF using input data file (both ascii and fits files are supported), use `tpcf` method as follows

    `correl, poserr=tpcf('/path/to/datfile.dat',bins, randfile='/path/to/randomfile.dat', weights=True)`

    If random file is not available or not provided, we can generate random catalog by providing the mangle mask file in `.ply` format along with specifying the size of the catalog in multiples of size of data catalog (default 2x size). To do this

    `correl, poserr=tpcf('/path/to/datfile.dat', bins, maskfile='/path/to/maskfile.ply', weights=True, randfact=3)`

    This returns `correl` and `poserr` `numpy` arrays corresponding to Two-point correlation and Poisson error

    ### Keyword Arguments
    The following keyword arguments can be included as needed

    #### Data file (Mandatory)

    Data file of galaxy/quasar redshift survey must be passed as the first argument to both `tpcf` and `atpcf` methods.

    **Supported filetypes**: ascii text files with columns, csv files or fits files are all supported. Most files provided by SDSS Value added catalogs should be directly usable.

     **To contain**: Any type of file provided must at least have columns named **Z** (redshift), **RA** (Right Ascension), **DEC** (Declination). These column names can be in any case.

     If one intends to use `weights=True` option (must to obtain accurate results) the data file must also contain radial weights with column title **radial_weight** or **WEIGHT_SYSTOT**

    #### bins (Mandatory)

    A numpy array with ascending values in $c/H_0$ units must be provided as the second argument to both `tpcf` and `atpcf` methods. In case of `atpcf` it automatically creates 2D bins as `bins2d=(bins,bins)` from provided 1D `bins`

    #### `randfile=` Path to random file (semi-Optional)

    If not provided, `maskfile=` argument must be given `.ply` file.

    **Supported filetypes**: ascii text files with columns, csv files or fits files are all supported. Most files provided by SDSS Value added catalogs should be directly usable.

     **To contain**: Any type of file provided must at least have columns named **Z** (redshift), **RA** (Right Ascension), **DEC** (Declination). These column names can be in any case.

     If one intends to use `weights=True` option (must to obtain accurate results) the data file must also contain radial weights with column title **radial_weight** or **WEIGHT_SYSTOT**

     **To add:** In future support for other column titles for weights will be added. To also add calculation of weights from n(z) and for random catalog generation.


    #### `mask=` Path to mangle polygon file (semi-Optional)

    If not provided, `randfile=` argument must be provided.

    **Supported filetypes**: `.ply` file containing Mangle polygons describing survey geometry in the standard format. Most files provided by SDSS Value added catalogs should be directly usable.

    #### `randfact=` (Optional)

    Size of the random catalog in integer multiples of size of data catalog if random catalog file is not provided. Default value is `2`

    #### `weights=` (Optional)

    It is highly recommended to use weights argument by providing `weights=True` to obtain accurate two-point correlation calculations. This picks up radial weights in the prescribed format (with column title **radial_weight** or **WEIGHT_SYSTOT** ) from the data and random files provided.

    If `weights=False`, by default *+1* will be added for each galaxy/random pair found within the bin instead of adding total weight. For more details on weights and references, see http://www.sdss3.org/dr9/tutorials/lss_galaxy.php

    #### `geometry='flat'` (Optional)

    **Available options**:

    `'flat'`(default) - for flat geometry of the Universe

     `'open'` - for Open Universe models like Milne

     `'close'` - for Closed Universe

     **Customization**

     Formulae for calculation of distances between two points (Z1, RA1, DEC1) and (Z2, RA2, DEC2) is taken from *T. Matsubara, Correlation function in deep redshift space as a cosmological probe, The Astrophysical Journal 615 (2) (2004) 573*. Using the formulae in this paper, distances squares (to reduce additional computational time distance squares are calculated to avoid using expensive `sqrt` function every time) are computed in the `metrics.pyx` file for all the above mentioned geometries. `Cython` is chosen for implementation to obtain faster results in building `BallTree`s  calculating `cdist` and to reduce `query` time.

     One can customize metric definitions as per one's need by editing this file. Also **K** (curvature parameter) in the formulae given in this reference need to be manually changed in the `metrics.pyx` for closed and open cases as per the model. After changing this compile it using `python metricsetup.py build_ext --inplace`

    #### `cosmology='lcdm'` (Optional)

    Used to calculate co-moving distances from redshifts.

    **Available options**:

    `'lcdm'` (default)- for Lambda CDM model

     `'lc'` - for $R_h=ct$ and linear coasting models

     **To add**: `wcdm` and other popular cosmology models soon

    #### `estimator=` (Optional)

    **Available options**:

    `'dp'` - Davis - Peebles estimator (default - fastest)

    `'ls'`- Landy - Szalay estimator

    `'ph'` - Peebles- Hauser estimator

    `'hew'` - Hewitt estimator

    `'h'` - Hamilton estimator

    For more details on estimator formulae see https://arxiv.org/pdf/1211.6211.pdf
    """
    # Default function arguments
    # weights = np.array([])
    weightsflag = False
    cosmology = 'lcdm'
    geometry = 'flat'
    metric = flatdistsq
    randcatfact = 2
    estimator = 'dp'
    binsq = bins**2
    randfile = None
    maskfile = None

    # Options for correl calculation estimators and cosmology models
    elist = ['dp', 'ls', 'ph', 'hew', 'h']
    clist = ['lcdm', 'lc']

    if kwargs is not None:
        for key, value in kwargs.iteritems():
            # print (key, value)
            if key.lower() == 'randfile':
                randfile = value
            elif key.lower() == 'randfact':
                randcatfact = value
            elif key.lower() == 'geometry':
                if value.lower() == 'flat':
                    geometry = 'flat'
                    metric = flatdistsq
                elif value.lower() == 'open':
                    geometry = 'open'
                    metric = opendistsq
                elif value.lower() == 'close':
                    geometry = 'close'
                    metric = closedistsq
                else:
                    print("Incorrect geometry argument provided! Using flat geometry")
            elif key.lower() == 'estimator':
                if value.lower() in elist:
                    estimator = value.lower()
                else:
                    print("Incorrect estimator provided! Using 'dp' as default")
            elif key.lower() == 'cosmology':
                if value.lower() in clist:
                    cosmology = value.lower()
                else:
                    print("Incorrect Cosmology provided! Using 'lcdm' as default")
            elif key.lower() == 'mask':
                maskfile = value
            elif key.lower() == 'weights':
                if value is True:
                    weightsflag = True
                    # fdat=readinfile(datfile,ftype='internal')
                    # weights=1.0/(1.0+4.0*np.array(fdat['nz']))
                    # weights=weights/np.mean(weights)
                    # print (weights)
                else:
                    weightsflag = False
            else:
                print ("key argument `%s` not valid" % key)
    else:
        print ("Refer documentation to enter valid keyword arguments")

    print("Calculating Correlation function with the following parameters")
    print ("data file=")
    print(datfile)
    print("random file=")
    print(randfile)
    print("Random catalog size factor(if random file is None)=")
    print(randcatfact)
    print("mask/window file=")
    print(maskfile)
    print ("Cosmology=")
    print(cosmology)
    print("Geometry=")
    print(geometry)
    print("Correl estimator=")
    print(estimator)
    print ("Weights=")
    print (weightsflag)
    print("-----------------------------------------")
    # Prepare dat from data file
    dat, weights = datprep(datfile, 'data', cosmology)
    global Nd
    Nd = len(dat)
    # Nd = len(dat)
    # print (weights)
    # Prepare datR from random file or generate a random catalog
    if randfile is None:
        randcatsize = randcatfact*Nd
        if maskfile is None:
            print ("Mask file compulsory. Please provide mask='maskfilepath.ply'")
        else:
            datR = randcatprep(datfile, randcatsize, maskfile, cosmology)
            rweights = np.array([])
            # randfile='./randcat.dat'
            # datR, rweights=datprep(randfile,'random',cosmology)
    else:
        datR, rweights = datprep(randfile, 'random', cosmology)

    # if len(weights)!=0:
        # rfdat=readinfile(randfile,ftype='internal')
        # rweights=1.0/(1.0+4.0*np.array(rfdat['nz']))
        # rweights=rweights/np.mean(rweights)
        # print (rweights)
    # Nr=len(datR)


    global Nr
    Nr = len(datR)


    #Creating module-wise global balltrees so that they don't have to be created many times.

    global dbt
    global rbt

    print ("Creating BallTree for data points using input metric...")
    dbt = BallTree(dat, metric='pyfunc', func=metric)

    print ("Creating BallTree for random points using input metric...")
    rbt = BallTree(datR, metric='pyfunc', func=metric)

    print ("Calculating 2pCF...")
    # f=(1.0*Nrd)/N
    # print (weights)
    # Reference: arXiv: 1211.6211
    if estimator == 'dp':
        if weightsflag is False or len(weights) != Nd:
            # print (weightsflag)
            # print(len(weights))
            # print(len(datR))
            DD = DDcalc(dat, binsq, metric)
            DR = DRcalc(dat, datR, binsq, metric)
        else:
            # if len(rweights)!=len(datR):
            DD = DDwcalc(dat, binsq, metric, weights)
            DR = RDwcalc(dat, datR, binsq, metric, weights)
            # else:
            #     DD=DDwcalc(dat,binsq,metric,weights)
            #     DR=DRwcalc(dat,datR,binsq,metric,rweights)
        print ("Using Davis-Peebles estimator")
        correl = (DD/DR)-1.0

    elif estimator == 'ph':
        if weightsflag is False or len(weights) != Nd:
            DD = DDcalc(dat, binsq, metric)
            RR = RRcalc(datR, binsq, metric)
        else:
            DD = DDwcalc(dat, binsq, metric, weights)
            RR = RRwcalc(datR, binsq, metric, rweights)
        print ("Using Peebles-Hauser estimator")
        correl = (DD/RR)-1.0
    else:
        if weightsflag == False or len(weights) != Nd:
            DD = DDcalc(dat, binsq, metric)
            RR = RRcalc(datR, binsq, metric)
            DR = DRcalc(dat, datR, binsq, metric)
        else:
            DD = DDwcalc(dat, binsq, metric, weights)
            RR = RRwcalc(datR, binsq, metric, rweights)
            DR = RDwcalc(dat, datR, binsq, metric, weights)
        if estimator == 'ls':
            print ("Using Landy-Szalay estimator")
            correl = (DD-2.0*DR+RR)/RR
        elif estimator == 'hew':
            print ("Using Hewett estimator")
            correl = (DD-DR)/RR
        elif estimator == 'h':
            print ("Using Hamilton estimator")
            correl = (DD*RR)/DR**2 - 1.0
    correlerr = poserr(correl, DD*Nd*(Nd-1.0))
    print("Two-point correlation=")
    print (correl, correlerr)
    return correl, correlerr


def DDcalc(dat, bins, metric):
    print ("Calculating DD...\n DD=")
    DD = autocorr(dat, bins, metric)
    DD [DD == 0] = 1.0
    # Nd = len(dat)
    DD = 2.0*DD/(Nd*(Nd-1.0))
    print (DD)
    return DD


def RRcalc(datR, bins, metric):
    print ("Calculating RR...\n RR=")
    RR = autocorr(datR, bins, metric)
    RR [RR == 0] = 1.0
    # Nr = len(datR)
    RR = 2.0*RR/(Nr*(Nr-1.0))
    print (RR)
    return RR


def DRcalc(dat, datR, bins, metric):
    print ("Calculating DR...\n DR=")
    DR = crosscorr(dat, datR, bins, metric)
    DR[DR == 0] = 1.0
    # Nd = len(dat)
    # Nr = len(datR)
    DR = DR/(Nd*Nr)
    print (DR)
    return DR


def autocorr(dat, bins, metric):
    counts_DD = dbt.two_point_correlation(dat, bins)
    DD = np.diff(counts_DD)
    return DD


def crosscorr(dat, datR, bins, metric):
    counts_DR = rbt.two_point_correlation(dat, bins)
    DR = np.diff(counts_DR)
    return DR


def poserr(xi, DD):
    print ("Calculating Poisson error")
    return (1.0+xi)/np.sqrt(DD)
# alternatively
# rbt=BallTree(dat,metric='pyfunc',func=metric)
# counts_RD=rbt.two_point_correlation(dat,bins)


def DDwcalc(dat, bins, metric, weights):
    print ("Calculating DD with weights (parallelized)...\n DD=")
    # DD = autocorrw(dat, bins, metric, weights)
    # Nd = len(dat)
    DD = multi_autocp(dat, bins, metric, weights, Nd, pcpus)
    DD[DD == 0] = 1.0
    DD = DD/(Nd*(Nd-1.0)) # factor of 2 cancels with 1/2 that needs to be done to remove double counting of pairs
    print (DD)
    return DD


def RRwcalc(datR, bins, metric, weights):
    print ("Calculating RR with weights (parallelized)...\n RR=")
    # RR = autocorrw(datR, bins, metric, weights)
    # Nr = len(datR)
    RR = multi_autocp(datR, bins, metric, weights, Nr, pcpus)
    RR[RR == 0] = 1.0

    RR = RR/(Nr*(Nr-1.0))
    print (RR)
    return RR


def DRwcalc(dat, datR, bins, metric, rweights):
    print ("Calculating DR with weights (parallelized)...\n DR=")
    # DR = crosscorrw(dat, datR, bins, metric, rweights)
    # Nd = len(dat)
    # Nr = len(datR)
    DR = multi_crosscp(dat, datR, bins, metric, rweights, Nd, pcpus)
    DR[DR == 0] = 1.0
    DR = DR/(Nd*Nr)
    print (DR)
    return DR


def RDwcalc(dat, datR, bins, metric, weights):
    print ("Calculating RD with weights...\n RD=")
    # DR = crosscorrwrd(dat, datR, bins, metric, weights)
    # Nd = len(dat)
    # Nr = len(datR)
    DR = multi_crosscp(dat, datR, bins, metric, weights, Nr, pcpus)
    DR[DR == 0] = 1.0
    DR = DR/(Nd*Nr)
    print (DR)
    return DR


def autocorrw(dat, bins, metric, weights):
    # dbt = BallTree(dat, metric='pyfunc', func=metric)
    DD = np.zeros(len(bins)-1)
    for i in tqdm(range(len(dat))):
        ind = dbt.query_radius(dat[i].reshape(1, -1), max(bins))
        # wts=np.array([])
        for j in ind:
            dist0 = dist.cdist([dat[i], ], dat[j], metric)[0]
            DD += np.histogram(dist0, bins=bins, weights=weights[j])[0]
            # print (dist0,weights[j])
    print (DD)
    return DD


def crosscorrw(dat, datR, bins, metric, rweights):
    # rbt = BallTree(datR, metric='pyfunc', func=metric)
    DR = np.zeros(len(bins)-1)
    for i in tqdm(range(len(dat))):
        ind = rbt.query_radius(dat[i].reshape(1, -1), max(bins))
        # wts=np.array([])
        for j in ind:
            dist0 = dist.cdist([dat[i], ], datR[j], metric)[0]
            DR += np.histogram(dist0, bins=bins, weights=rweights[j])[0]
            # print (dist0,weights[j])
    return DR


def crosscorrwrd(dat, datR, bins, metric, weights):
    # dbt = BallTree(dat, metric='pyfunc', func=metric)
    RD = np.zeros(len(bins)-1)
    # p=multiprocessing.Pool(processes=multiprocessing.cpu_count())
    # RD=p.map(rdcalc, range(len(datR)))
    for i in tqdm(range(len(datR))):
    # def rdcalc():
        ind = dbt.query_radius(datR[i].reshape(1, -1), max(bins))
        #  wts=np.array([])
        for j in ind:
            dist0 = dist.cdist([datR[i], ], dat[j], metric)[0]
            RD += np.histogram(dist0, bins=bins, weights=weights[j])[0]
                # print (dist0,weights[j])
            # return RD
    print(RD)
    return RD


def autocorrwp(dat, bins, metric, weights, rNd, multi=False, queue=0):
    # dbt = BallTree(dat, metric='pyfunc', func=metric)
    DD = np.zeros(len(bins)-1)
    for i in tqdm(rNd):
        ind = dbt.query_radius(dat[i].reshape(1, -1), max(bins))
        # wts=np.array([])
        for j in ind:
            dist0 = dist.cdist([dat[i], ], dat[j], metric)[0]
            DD += np.histogram(dist0, bins=bins, weights=weights[j])[0]
            # print (dist0,weights[j])
    if multi:
        queue.put(DD)
    else:
        return DD
    print (DD)
    return DD


def crosscorrwrdp(dat, datR, bins, metric, weights, rNr, multi=False, queue=0):
    # dbt = BallTree(dat, metric='pyfunc', func=metric)
    RD = np.zeros(len(bins)-1)
    # p=multiprocessing.Pool(processes=multiprocessing.cpu_count())
    # RD=p.map(rdcalc, range(len(datR)))
    for i in tqdm(rNr):
    # def rdcalc():
        ind = dbt.query_radius(datR[i].reshape(1, -1), max(bins))
        #  wts=np.array([])
        for j in ind:
            dist0 = dist.cdist([datR[i], ], dat[j], metric)[0]
            RD += np.histogram(dist0, bins=bins, weights=weights[j])[0]
                # print (dist0,weights[j])
            # return RD
    if multi:
        queue.put(RD)
    else:
        return RD
    print(RD)
    return RD


def multi_autocp(dat, bins, metric, weights, Nd, CORES=pcpus):

    DD = np.zeros(len(bins)-1)
    queues = [RetryQueue() for i in range(CORES)]
    args = [(dat, bins, metric, weights, range(int(Nd*i/CORES),int(Nd*(i+1)/CORES)), True, queues[i]) for i in range(CORES)]
    jobs = [Process(target=autocorrwp, args=(a)) for a in args]
    for j in jobs: j.start()
    for q in queues: DD+=q.get()
    for j in jobs: j.join()

    return DD


def multi_crosscp(dat, datR, bins, metric, weights, Nr, CORES=pcpus):

    DR = np.zeros(len(bins)-1)
    queues = [RetryQueue() for i in range(CORES)]
    args = [(dat, datR, bins, metric, weights, range(int(Nr*i/CORES),int(Nr*(i+1)/CORES)), True, queues[i]) for i in range(CORES)]
    jobs = [Process(target=crosscorrwrdp, args=(a)) for a in args]
    for j in jobs: j.start()
    for q in queues: DR+=q.get()
    for j in jobs: j.join()

    return DR

def retry_on_eintr(function, *args, **kw):
    while True:
        try:
            return function(*args, **kw)
        except IOError, e:
            if e.errno == KeyboardInterrupt:
                break
            elif e.errno == errno.EINTR:
                continue
            else:
                raise

class RetryQueue(Queue):

    def get(self, block=True, timeout=None):
        return retry_on_eintr(Queue.get, self, block, timeout)



#!/usr/bin/env python
# import sys
from setuptools import setup
from distutils.core import setup
from distutils.extension import Extension

def readme():
        with open('README.rst') as f:
                    return f.read()
try:
    from Cython.Distutils import build_ext
    from Cython.Build import cythonize
except ImportError:
    use_cython = False
else:
    use_cython = True

cmdclass = {}
# ext_modules = [cythonize('*.pyx') ]
ext_modules = []

if use_cython:
    ext_modules += [Extension("correlcalc.metrics", ["metrics/metrics.pyx"]), ]
    cmdclass.update({'build_ext': build_ext})
else:
    ext_modules += [Extension("correlcalc.metrics", ["metrics/metrics.c"]), ]
setup(
    name='correlcalc',
    version='0.974rc1',
    description='Two-point correlation function (2pCF) calculation',
    long_description=readme(),
    url='http://github.com/rohinkumar/correlcalc',
    author='Rohin Kumar Y',
    author_email='yrohinkumar@gmail.com',
    license='MIT',
    packages=['correlcalc'],
    # ,'correlcalc.metrics'
    # package_dir={
    #     'correlcalc' : base_dir + '/correlcalc',
    # },
    install_requires=['numpy', 'scipy', 'astropy', 'cython', 'tqdm', 'pymangle', 'sklearn'],
    # matplotlib
    # dependency_links=['https://github.com/esheldon/healpix_util/master/tarball#egg=package-0.1'],
    cmdclass = cmdclass,
    ext_modules=ext_modules,
    include_package_data=True,
    zip_safe=False)



__author__ = 'Rohin Kumar Y'
# import numpy as np
# import math as m
# from scipy import integrate
# from .comovdist import *
# from .metrics import metrics
# from .
#from metrics.metrics import *
global Om
global Ol

# global datpath
# global randpath
# global maskpath

from .tpcf import *
# from .antpcf import *
from .datprep import *
from .antpcf import *
#from .datvis import *
# try:
#    from correlcalc.metrics import *
# except ImportError:
#    print "correlcalc.metrics ImportError"
# from metrics import *
# from hw import hw
# def version():
#    print "correlcalc version no. 0.93"



__author__ = 'Rohin Kumar Y'
import os
import astropy.io.ascii as ascii
import astropy.io.fits as fits
from astropy.table import Table
import pymangle


def readinfile(filename, ftype):
    """Method to run basic checks on input data/random files and return contents"""
    if os.path.isfile(filename):
        dat = ascii.read(filename)
        cols = ['z', 'ra', 'dec']
        colnames = []
        for x in dat.colnames:
            colnames.append(x.lower())
        if all(x in colnames for x in cols):
            if ftype.lower() == 'data':
                print ("Entered ascii data file")
                print (dat)
                return dat
            elif ftype.lower() == 'random':
                print ("Entered ascii random file")
                print (dat)
                return dat
            elif ftype.lower() == 'internal':
                return dat
            else:
                print ("Please provide the file as 'data' or 'random'")
                return None
        else:
            print ("File should at least contain columns named 'z', 'ra' and 'dec'. Column names in your ascii file are")
            print(colnames)
            return None
    else:
        print ("Invalid File Path, File Doesn't exist")
        return None


def readfitsfile(fname, ftype):
    """Basic checks for fits file on input data/random files and returns table data from fits"""
    if os.path.isfile(fname):
        hdu_list = fits.open(fname, memmap=True)
        print("Entered fits file containing following data")
        hdu_list.info()
        print(hdu_list[1].columns)
        dat = Table(hdu_list[1].data)
        cols = ['z', 'ra', 'dec']
        colnames = []
        for x in dat.colnames:
            colnames.append(x.lower())
        if all(x in colnames for x in cols):
            if ftype.lower() == 'data':
                print ("Entered fits data file")
                print (dat)
                return dat
            elif ftype.lower() == 'random':
                print ("Entered fits random file")
                print (dat)
                return dat
            elif ftype.lower() == 'internal':
                return dat
            else:
                print ("Please provide the file as 'data' or 'random'")
                return None
        else:
            print ("File should at least contain columns named 'z', 'ra' and 'dec'. Column names in your fits file are")
            print(colnames)
            return None
    else:
        print ("Invalid File Path, File Doesn't exist")
        return None
    # else:
    #         print ("Please provide fits file with .fits extension")
    #         return None


def readmaskfile(fname):
    """Basic checks for mangle ply file. Returns mangle object"""
    if os.path.isfile(fname):
        if fname.lower().endswith('.ply'):
            mangle = pymangle.Mangle(fname)
            print(mangle)
            return mangle
        else:
            print ("Please provide mangle polygon file with .ply extension")
            return None
    else:
        print ("Invalid File Path, File Doesn't exist")
        return None


def storerandcat(z, ra, dec, rcatname):
    """Create random catalog file to store generated random catalog from randcatprep method"""
    fobj = open(rcatname, 'w')
    fobj.write("z\t ra\t dec\n ")
    for i in range(0, len(z)):
        fobj.write("%f\t " % z[i])
        fobj.write("%f\t %f\n " % (ra[i], dec[i]))
    fobj.close()



__author__ = 'Rohin Kumar Y'
from scipy.stats import gaussian_kde
import numpy as np
from fileios import *

def kde(x, x_grid, bandwidth=0.2, **kwargs):
    """Kernel Density Estimation with Scipy"""
    kde = gaussian_kde(x, bw_method=bandwidth / x.std(ddof=1), **kwargs)
    return kde.evaluate(x_grid)

def generate_rand_from_pdf(pdf, x_grid, N):
    """Method to generate 'N' no. of random numbers from input probability distribution function (pdf) in form of kernel density"""
    cdf = np.cumsum(pdf)
    cdf = cdf / cdf[-1]
    values = np.random.rand(N)
    value_bins = np.searchsorted(cdf, values)
    random_from_cdf = x_grid[value_bins]
    return random_from_cdf

def randang(maskfile,randcatsize):
    """Method to calculate RA and DEC from mangle .ply file"""
    mangle=readmaskfile(maskfile)
    rar,decr=mangle.genrand(randcatsize)
    return rar,decr

def randz(z,randcatsize):
    """Method to calculate random redshift values from input redshift distribution"""
    x_grid=np.linspace(min(z),max(z),randcatsize)
    pdf=kde(z,x_grid,bandwidth=1e-3)
    return generate_rand_from_pdf(pdf, x_grid, randcatsize)
